%ifarch aarch64
%global bootcsv BOOTAA64.CSV
%endif

%ifarch x86_64
%global bootcsv BOOTX64.CSV
%endif

%global debug_package %{nil}
%undefine _debuginfo_subpackages

%global enable_sm 0
%global vendor_cert %{nil}

Summary:       First-stage UEFI bootloader
Name:          shim
Version:       15.7
Release:       9%{?dist}
License:       BSD
URL:           https://github.com/rhboot/shim
Source0:       %{url}/releases/download/%{version}/shim-%{version}.tar.bz2
Source1:       BOOTAA64.CSV
Source2:       BOOTX64.CSV

Patch0001:     backport-0001-CVE-2023-40546-mok-fix-LogError-invocation.patch
Patch0002:     backport-0002-Add-primitives-for-overflow-checked-arithmetic-opera.patch
Patch0003:     backport-0003-CVE-2023-40551-pe-relocate-Fix-bounds-check-for-MZ-b.patch
Patch0004:     backport-0004-pe-relocate-make-read_header-use-checked-arithmetic-.patch
Patch0005:     backport-0005-CVE-2023-40550-pe-Fix-an-out-of-bound-read-in-verify.patch
Patch0006:     backport-0006-pe-relocate-Ensure-nothing-else-implements-CVE-2023-.patch
Patch0007:     backport-0007-CVE-2023-40549-Authenticode-verify-that-the-signatur.patch
Patch0008:     backport-0008-CVE-2023-40548-Fix-integer-overflow-on-SBAT-section-.patch
Patch0009:     backport-0009-Further-mitigations-against-CVE-2023-40546-as-a-clas.patch
Patch0010:     backport-0010-CVE-2023-40547-avoid-incorrectly-trusting-HTTP-heade.patch

# Feature for shim SMx support
Patch3000:     openssl-add-ecdsa-and-ec-support-for-shim.patch
Patch3001:     shim-support-sm2-and-sm3-algorithm.patch

BuildRequires: elfutils-libelf-devel openssl-devel openssl git pesign gnu-efi gnu-efi-devel gcc
%ifarch aarch64
BuildRequires: binutils
%endif
Requires:      efi-filesystem mokutil
Provides:      bundled(openssl) = 1.0.2j
Provides:      shim-%{efi_arch} = %{version}-%{release}

ExclusiveArch:     x86_64 aarch64

%description
Initial UEFI bootloader that handles chaining to a trusted full bootloader under secure boot environments.

%prep
%autosetup -n shim-%{version} -p1 -S git
mkdir build-%{efi_arch}
sed -i 's/vendor/%{_vendor}/g' %{_sourcedir}/BOOTX64.CSV
sed -i 's/vendor/%{_vendor}/g' %{_sourcedir}/BOOTAA64.CSV

%build
COMMITID=$(cat commit)
MAKEFLAGS="TOPDIR=.. -f ../Makefile COMMITID=${COMMITID} "
MAKEFLAGS+="EFIDIR=%{_vendor} PKGNAME=shim RELEASE=%{release} "
MAKEFLAGS+="ENABLE_HTTPBOOT=true ENABLE_SHIM_HASH=true "
%if 0%{enable_sm}
	MAKEFLAGS+="ENABLE_SHIM_SM=true "
%endif
%if "%{vendor_cert}" != ""
	MAKEFLAGS+="VENDOR_CERT_FILE+=%{vendor_cert} "
%endif
MAKEFLAGS+="%{_smp_mflags}"

cd build-%{efi_arch}
make ${MAKEFLAGS} DEFAULT_LOADER='\\\\grub%{efi_arch}.efi' all

%install
COMMITID=$(cat commit)
MAKEFLAGS="TOPDIR=.. -f ../Makefile COMMITID=${COMMITID} "
MAKEFLAGS+="EFIDIR=%{_vendor} PKGNAME=shim RELEASE=%{release} "
MAKEFLAGS+="ENABLE_HTTPBOOT=true ENABLE_SHIM_HASH=true "

cd build-%{efi_arch}
make ${MAKEFLAGS} \
    DEFAULT_LOADER='\\\\grub%{efi_arch}.efi' \
    DESTDIR=%{buildroot} \
    install-debuginfo install-debugsource

install -D -d -m 0755 %{buildroot}/boot/
install -D -d -m 0700 %{buildroot}%{efi_esp_root}/
install -D -d -m 0700 %{buildroot}%{efi_esp_efi}/
install -D -d -m 0700 %{buildroot}%{efi_esp_dir}/
install -D -d -m 0700 %{buildroot}%{efi_esp_boot}/
install -m 0700 shim%{efi_arch}.efi   %{buildroot}%{efi_esp_dir}/shim%{efi_arch}.efi
install -m 0700 mm%{efi_arch}.efi     %{buildroot}%{efi_esp_dir}/mm%{efi_arch}.efi
install -m 0700 %{bootcsv}            %{buildroot}%{efi_esp_dir}/BOOT%{efi_arch_upper}.CSV
install -m 0700 shim%{efi_arch}.efi   %{buildroot}%{efi_esp_boot}/BOOT%{efi_arch_upper}.EFI
install -m 0700 fb%{efi_arch}.efi     %{buildroot}%{efi_esp_boot}/fb%{efi_arch}.efi

# install the debug symbols
install -d %{buildroot}/usr/lib/debug/%{efi_esp_dir}
install -m 644 fb%{efi_arch}.efi.debug %{buildroot}/usr/lib/debug/%{efi_esp_dir}
install -m 644 mm%{efi_arch}.efi.debug %{buildroot}/usr/lib/debug/%{efi_esp_dir}
install -m 644 shim%{efi_arch}.efi.debug %{buildroot}/usr/lib/debug/%{efi_esp_dir}

rm -rf %{buildroot}/usr/lib/debug
rm -rf %{buildroot}/usr/src/debug

%files
%license COPYRIGHT
%{efi_esp_dir}/*%{efi_arch}*.efi
%{efi_esp_dir}/BOOT%{efi_arch_upper}.CSV
%{efi_esp_boot}/*%{efi_arch}.efi
%{efi_esp_boot}/*%{efi_arch_upper}.EFI

%changelog
* Tue Mar 18 2025 Anakin Zhang <anakinzhang@tencent.com> - 15.7-9
- Rebuilt for TencentOS 4

* Mon Mar 17 2025 Sinong Chen <costinchen@tencent.com> - 15.7-8
- Rebuilt for TencentOS 4

* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 15.7-7
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 15.7-6
- Rebuilt for loongarch release

* Tue Feb 20 2024 Yi Lin <nilusyi@tencent.com> - 15.7-5
- fix CVE-2023-40546, CVE-2023-40551, CVE-2023-40550, 
  CVE-2023-40549, CVE-2023-40548, CVE-2023-40547

* Thu Nov 16 2023 Xin Cheng <denisecheng@tencent.com> - 15.7-4
- replace vendor in bootcsv with %{_vendor}

* Tue Oct 10 2023 Feng Weiyao <wynnfeng@tencent.com> - 15.7-3
- add patches to support SMx

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 15.7-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 01 2023 kianli <kianli@tencent.com> - 15.7-1
- Upgrade to 15.7

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 15.6-5
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 15.6-4
- Rebuilt for OpenCloudOS Stream 23

* Tue Feb 7 2023 Shuo Wang <abushwang@tencent.com> - 15.6-3
- remove debuginfo and debugsource

* Wed Jan 11 2023 Shuo Wang <abushwang@tencent.com> - 15.6-2
- source build shim

* Tue Sep 20 2022 Shuo Wang <abushwang@tencent.com> - 15.6-1
- initial build

